package at.ac.uibk.gitsearch.repository.search;

import static org.hamcrest.Matchers.contains;
import static org.hamcrest.Matchers.hasProperty;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.IOException;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import javax.ws.rs.NotFoundException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.node.NodeValidationException;
import org.hamcrest.MatcherAssert;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestConfiguration;
import at.ac.uibk.gitsearch.service.dto.AutoCompleteEntry;

@SpringBootTest(classes = GitsearchApp.class)
public class MetaDataRepositoryIT {

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(MetaDataRepositoryIT.class);

	@Autowired
	RestHighLevelClient elasticsearchTestClient;

	@Autowired
	ElasticSearchTestConfiguration elasticSearchTestConfiguration;
	
	static ElasticSearchTestConfiguration staticElasticSearchTestConfiguration;

	@BeforeEach // must be started as BeforeEach, in order to autowire the ElasticSearchTestConfiguration
	@Timeout(value = 2, unit = TimeUnit.MINUTES)
	public  void setUpESServer() throws IOException, NodeValidationException {
		if(staticElasticSearchTestConfiguration == null) {
			elasticSearchTestConfiguration.startTestNode();
			staticElasticSearchTestConfiguration = elasticSearchTestConfiguration;
		}
	}
	
	@AfterAll
	public static void shutDownESServer() {
		if(staticElasticSearchTestConfiguration!=null) {
			staticElasticSearchTestConfiguration.stopTestContainer();
			staticElasticSearchTestConfiguration = null;
		}
	}

	@BeforeEach
	public void setUpIndex() {
	}

	@Autowired
	private MetaDataRepository metaDataRepository;

	@Test
	public void testKeywordAutocompletion() throws IOException {
		final List<AutoCompleteEntry> keywordsAutoComplete = metaDataRepository.getKeywordsAutoComplete("Jav", 10);
		MatcherAssert.assertThat(keywordsAutoComplete, contains(hasProperty("target", is("Java"))));
	}

	@Test
	public void testCreatorAutocompletion() throws IOException {
		final List<AutoCompleteEntry> creatorAutoComplete = metaDataRepository.getCreatorAutoComplete("Pod", 10);
		MatcherAssert.assertThat(creatorAutoComplete, contains(hasProperty("target", is("Stefan Podlipnig"))));
		final List<AutoCompleteEntry> creatorAutoComplete2 = metaDataRepository.getCreatorAutoComplete("Po", 10);
		MatcherAssert.assertThat(creatorAutoComplete2, contains(hasProperty("target", is("Stefan Podlipnig"))));
	}

	@Test
	public void testContributorAutocompletion() throws IOException {
		final List<AutoCompleteEntry> contributorAutoComplete = metaDataRepository.getContributorAutoComplete("Bast", 10);
		MatcherAssert.assertThat(contributorAutoComplete, contains(hasProperty("target", is("Daniel Bastta"))));
	}

	@Test
	public void testProgrammingLanguageAutocompletion() throws IOException {
		final List<AutoCompleteEntry> plAutoComplete = metaDataRepository.getProgrammingLanguageAutoComplete("Ja", 10);
		MatcherAssert.assertThat(plAutoComplete, contains(hasProperty("target", is("JAVA"))));
	}

	@Test
	public void getByExerciseId() throws IOException {
		// This test assumes that the test entries are public
		for (int i : new int[] { 1 }) {
			final SearchResultDTO exerciseById = metaDataRepository.getExerciseById(i + "", Optional.empty());
			assertNotNull("should be found:", exerciseById);
			assertEquals("Id should be the same", i + "", exerciseById.getExerciseId());
		}
	}

	@Test()
	public void getByExerciseIdNegative() throws IOException {
		// This test assumes that the test entries have consecutive ids
		String exerciseId = "999999"; // should not exist
		Assertions.assertThrows(NotFoundException.class, () -> {
			metaDataRepository.getExerciseById(exerciseId, Optional.empty());},
				"Exercise for id " + exerciseId + " should not exist:");
	}
}
