package at.ac.uibk.gitsearch.service.dto;

import static org.junit.Assert.assertEquals;

import java.lang.reflect.InvocationTargetException;

import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.codeability.sharing.plugins.api.search.SearchResultDTO.GitProject;
import org.codeability.sharing.plugins.api.search.SearchResultsDTO;
import org.codeability.sharing.plugins.api.search.UserProvidedMetadataDTO;
import org.codeability.sharing.plugins.api.search.UserProvidedMetadataDTO.Person;
import org.junit.Assert;
import org.junit.jupiter.api.Test;

import at.ac.uibk.gitsearch.properties.ApplicationProperties.DeploymentInfo;
import at.ac.uibk.gitsearch.service.MailService;
import at.ac.uibk.gitsearch.testingUtilities.PropertiesTester;
import nl.jqno.equalsverifier.EqualsVerifier;
import nl.jqno.equalsverifier.Warning;

/**
 * Integration tests for {@link MailService}.
 */
public class VariousDTOTest {

	PropertiesTester propertiesTester = new PropertiesTester();

	@Test
	public void testUserProvidedMetadataDTO() throws IllegalAccessException, InvocationTargetException {


		propertiesTester.testProperties(UserProvidedMetadataDTO.class);
		UserProvidedMetadataDTO testUPMD = new UserProvidedMetadataDTO();
		UserProvidedMetadataDTO testUPMDclone = new UserProvidedMetadataDTO(testUPMD);
		Assert.assertEquals("clone should be equas", testUPMD, testUPMDclone);
		EqualsVerifier.forClass(UserProvidedMetadataDTO.class).suppress(Warning.NONFINAL_FIELDS).verify();

	}

	@Test
	public void testMessageDTO() {
		MessageDTO mess = new MessageDTO("someMessage");
		assertEquals("someMessage", mess.getMessage());
		propertiesTester.testProperties(MessageDTO.class);
	}

	@Test
	public void testMetadataUserDTO() {
		MetadataUserDTO user = new MetadataUserDTO("name", "affiliation", "abc@def.de");
		assertEquals("name", user.getName());
		assertEquals("affiliation", user.getAffiliation());
		assertEquals("abc@def.de", user.getEmail());
		propertiesTester.testProperties(MetadataUserDTO.class);
	}

	@Test
	public void testSearchResultsDTO() throws IllegalAccessException, InvocationTargetException {
		propertiesTester.testProperties(SearchResultsDTO.class);
		propertiesTester.testProperties(SearchResultDTO.class);
		propertiesTester.testProperties(Person.class);
		propertiesTester.testProperties(GitProject.class);

		// just for test coverage
		@SuppressWarnings("unused")
		SearchResultsDTO srDTO = new SearchResultsDTO(null, 0, 0);
	}

    // does not work with metadata version 0.4
//	@Test
//	public void testSearchResultsEnums() throws IllegalAccessException, InvocationTargetException {
//		for(ExerciseType et: ExerciseType.values()) {
//			et.getExternalName();
//		}
//
//		for(GitProjectVisibility vis: GitProjectVisibility.values()) {
//			vis.getExternalName();
//		}
//	}

	// @org.junit.jupiter.api.Test
	// public void testSearchInputDTO() throws IllegalAccessException, InvocationTargetException {
	// 	propertiesTester.testProperties(SearchInputDTO.class);
	// }

	@org.junit.jupiter.api.Test
	public void testDeploymentInfo() throws IllegalAccessException, InvocationTargetException {
		propertiesTester.testProperties(DeploymentInfo.class);
	}

	@org.junit.jupiter.api.Test
	public void testVariousEquals() throws IllegalAccessException, InvocationTargetException {
		GitProject p1 = new GitProject(); p1.setProjectId(10);
		GitProject p2 = new GitProject(); p2.setProjectId(10);
		GitProject p3 = new GitProject(); p3.setProjectId(20);

		Assert.assertEquals(p1, p2);
		Assert.assertNotEquals(p1, p3);
		EqualsVerifier.forClass(GitProject.class)
		    .suppress(Warning.NONFINAL_FIELDS)
		    .withIgnoredFields("project_name", "namespace","main_group", "last_activity_at", "url", "sub_group").verify();
		EqualsVerifier.forClass(Person.class).suppress(Warning.NONFINAL_FIELDS).verify();
	}

}
