package at.ac.uibk.gitsearch.web.rest;

import static org.junit.Assert.assertEquals;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.properties.ApplicationProperties;
import at.ac.uibk.gitsearch.service.dto.BroadCastMessageDTO;

/**
 * Integration tests for the {@link UserResource} REST controller.
 */
@AutoConfigureMockMvc
@SpringBootTest(classes = GitsearchApp.class)
public class ApplicationInfoResourceIT {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(ApplicationInfoResourceIT.class);

    @Autowired
    private MockMvc restMockMvc;



    @Test
    public void testGetDeploymentInfo() throws Exception {
    	restMockMvc.perform(get("/api/applicationInfo/deploymentInfo")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
 	   .andExpect(content().contentType(MediaType.APPLICATION_JSON))
       .andExpect(status().isOk())
		.andExpect(TestUtil.testResult(ApplicationProperties.DeploymentInfo.class, r -> {
			assertEquals("This may fail, if config changes", r.getCommitId(), "Test Commit Id");
			assertEquals("This may fail, if config changes", r.getBranch(), "Test Branch");
		}));
    }

    @Test
    public void testGetBroadCastMessages() throws Exception {
    	restMockMvc.perform(get("/api/applicationInfo/broadcastMessages")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
 	   .andExpect(content().contentType(MediaType.APPLICATION_JSON))
       .andExpect(status().isOk())
		.andExpect(TestUtil.testResult(BroadCastMessageDTO[].class, r -> {
			LOGGER.info("There are currently {} messages", r.length);
		}));
    }



}
