import { TestBed, getTestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import { MessageService, BroadCastMessage } from 'app/shared/service/message-service';

describe('Service Tests', () => {
  describe('Message Service', () => {
    let injector: TestBed;
    let service: MessageService;
    let httpMock: HttpTestingController;
    let expectedResult: Array<BroadCastMessage>;
    let returnedFromService: Array<BroadCastMessage>;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
      });
      expectedResult = new Array<BroadCastMessage>();
      injector = getTestBed();
      service = injector.get(MessageService);
      httpMock = injector.get(HttpTestingController);

        const messageNow :BroadCastMessage  = {
              message: "Some Test now Message",
              starts_at: new Date(),
              ends_at: new Date(new Date().getDate() + 1),

              color: "red", id: 1, font: "unused", active: true, target_path: "unused", broadcast_type: "broadcast_type", dismissable:true
              
        };
        const messageTomorrow :BroadCastMessage  = {
              message: "Some Test tomorrwo Message",
              starts_at: new Date(new Date().getDate() + 1),
              ends_at: new Date(new Date().getDate() + 2),

              color: "red", id: 2, font: "unused", active: true, target_path: "unused", broadcast_type: "broadcast_type", dismissable:true
        }
        
        returnedFromService = new Array<BroadCastMessage>(messageNow, messageTomorrow);

    });
    
    

    describe('Service methods', () => {
      it('should get some messages', () => {

        expectedResult = service.getMessages(); // this may return an empty list, because webclient may not yet received results

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expectedResult = service.getMessages();
        expect(expectedResult).toMatchObject(returnedFromService);
      });


    });

    describe('Service methods', () => {
      it('should get some messages', () => {

        expectedResult = service.getActiveMessages(); // this may return an empty list, because webclient may not yet received results


        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expectedResult = service.getActiveMessages();
        const now = new Date();
        expect(expectedResult).toMatchObject(returnedFromService.filter(m => m.starts_at <= now && m.ends_at >= now));
      });


    });

    afterEach(() => {
      httpMock.verify();
    });
  });
});
